from sqlalchemy import *
from migrate import *
from dictionaries.versions import views_helper

def getTablesForSchemeCode(meta):
    barcodes1 = Table('barcodes1', meta, autoload=True)
    barcodes2 = Table('barcodes2', meta, autoload=True)
    prices1 = Table('prices1', meta, autoload=True)
    prices2 = Table('prices2', meta, autoload=True)
    return [barcodes1, barcodes2, prices1, prices2]

def getAspectValueSetTables(meta):
    aspectvalueset1 = Table('aspectvalueset1', meta, autoload=True)
    aspectvalueset2 = Table('aspectvalueset2', meta, autoload=True)
    return [aspectvalueset1, aspectvalueset2]

def getAspectTables(meta):
    aspect1 = Table('aspect1', meta, autoload=True)
    aspect2 = Table('aspect2', meta, autoload=True)
    return [aspect1, aspect2]

def upgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    for tbl in getTablesForSchemeCode(meta):
        if not 'aspectvaluesetcode' in tbl.columns:
            Column('aspectvaluesetcode', Integer, nullable=True).create(tbl)
        
    for tbl in getAspectValueSetTables(meta):
        if not 'remain' in tbl.columns:
            Column('remain', Numeric(13, 3), nullable=True).create(tbl)
        
    for tbl in getAspectTables(meta):
        if not 'aspectvaluedesc' in tbl.columns:
            Column('aspectvaluedesc', Integer, ColumnDefault(0)).create(tbl)        
        
    for v in ['barcodes', 'prices', 'aspectvalueset', 'aspect']:
        views_helper.update_dic_view(migrate_engine, v)    

def downgrade(migrate_engine):
    meta = MetaData(bind = migrate_engine)
    
    views = {}
    for v in ['barcodes', 'prices', 'aspectvalueset', 'aspect']:
        views[v] = views_helper.get_table_name(migrate_engine,v)
        
    for tbl in getTablesForSchemeCode(meta):
        if 'aspectvaluesetcode' in tbl.columns:
            tbl.c.aspectvaluesetcode.drop()    
    
    for tbl in getAspectValueSetTables(meta):
        if 'remain' in tbl.columns:
            tbl.c.remain.drop()

    for tbl in getAspectTables(meta):
        if 'aspectvaluedesc' in tbl.columns:
            tbl.c.aspectvaluedesc.drop()
                
    for v in views.keys():
        views_helper.update_dic_view(migrate_engine, v, views[v])
        