from sqlalchemy import Table, MetaData, Index
from dictionaries.versions import views_helper

def getTmc(meta):
    tmc1 = Table('tmc1', meta, autoload=True)
    tmc2 = Table('tmc2', meta, autoload=True)
    return [tmc1, tmc2]

def getPrices(meta):
    prices1 = Table('prices1', meta, autoload=True)
    prices2 = Table('prices2', meta, autoload=True)
    return [prices1, prices2]

def getAspectValueSet(meta):
    aspectvalueset1 = Table('aspectvalueset1', meta, autoload=True)
    aspectvalueset2 = Table('aspectvalueset2', meta, autoload=True)
    return [aspectvalueset1, aspectvalueset2]

def upgrade(migrate_engine):

    meta = MetaData(bind = migrate_engine)

    for tbl in getTmc(meta):
        if 'tmcidxprice' not in [ind.name for ind in tbl.indexes]:
            Index('tmcidxprice', tbl.c.price).create()
        if 'tmcidxaspectvaluesetcode' not in [ind.name for ind in tbl.indexes]:
            Index('tmcidxaspectvaluesetcode', tbl.c.aspectvaluesetcode).create()
        if 'tmcidxaspectschemecode' not in [ind.name for ind in tbl.indexes]:
            Index('tmcidxaspectschemecode', tbl.c.aspectschemecode).create()

    for tbl in getPrices(meta):
        if 'pricesidxprice' not in [ind.name for ind in tbl.indexes]:
            Index('pricesidxprice', tbl.c.price).create()

    for tbl in getAspectValueSet(meta):
        if 'aspectvsidxprice' not in [ind.name for ind in tbl.indexes]:
            Index('aspectvsidxprice', tbl.c.price).create()

    views_helper.update_dic_view(migrate_engine, 'tmc')
    views_helper.update_dic_view(migrate_engine, 'prices')
    views_helper.update_dic_view(migrate_engine, 'aspectvalueset')

def downgrade(migrate_engine):

    meta = MetaData(bind = migrate_engine)
    
    for tbl in getTmc(meta):
        if 'tmcidxprice' in [ind.name for ind in tbl.indexes]:
            Index('tmcidxprice', tbl.c.price).drop()
        if 'tmcidxaspectvaluesetcode' in [ind.name for ind in tbl.indexes]:
            Index('tmcidxaspectvaluesetcode', tbl.c.aspectvaluesetcode).drop()
        if 'tmcidxaspectschemecode' in [ind.name for ind in tbl.indexes]:
            Index('tmcidxaspectschemecode', tbl.c.aspectschemecode).drop()

    for tbl in getPrices(meta):
        if 'pricesidxprice' in [ind.name for ind in tbl.indexes]:
            Index('pricesidxprice', tbl.c.price).drop()

    for tbl in getAspectValueSet(meta):
        if 'aspectvsidxschemecode' in [ind.name for ind in tbl.indexes]:
            Index('aspectvsidxschemecode', tbl.c.aspectschemecode).drop()
        if 'aspectvsidxprice' in [ind.name for ind in tbl.indexes]:
            Index('aspectvsidxprice', tbl.c.price).drop()

    views_helper.update_dic_view(migrate_engine, 'tmc')
    views_helper.update_dic_view(migrate_engine, 'prices')
    views_helper.update_dic_view(migrate_engine, 'aspectvalueset')
