# -*- coding: utf-8 -*-
from sqlalchemy import *
from migrate import *
from sqlalchemy.dialects.mysql.pyodbc import MySQLDialect
from documents.versions import dump

def dropTriggers(migrate_engine):
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterInsert`;")
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterUpdate`;")
    migrate_engine.connect().execute("DROP TRIGGER IF EXISTS `triggerAfterDelete`;")

def createTriggers(migrate_engine):
    trigger_after_insert = """
CREATE TRIGGER triggerAfterInsert AFTER INSERT ON document
FOR EACH ROW BEGIN
    SET @factor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, new.sumb, 0),
        sumsalecash = COALESCE(sumsalecash, 0) + if(new.doctype = 1, new.sumcash, 0),
        sumsalenoncash = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, new.sumnoncash, 0),
        sumsaleother = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, new.sumother, 0),
        sumgaincash = COALESCE(sumgaincash, 0) + if(new.doctype in (1,2,25), new.sumcash * @factor, 0),
        sumgainnoncash = COALESCE(sumgainnoncash, 0) + if(new.doctype in (1,2,25), new.sumnoncash * @factor, 0),
        sumrefund = COALESCE(sumrefund, 0) + if(new.doctype in (2,25), abs(new.sumb), 0),
        sumrefundcash = COALESCE(sumrefundcash, 0) + if(new.doctype in (2,25), new.sumcash, 0),
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) + if(new.doctype in (2,25), new.sumnoncash, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), new.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) + (new.sumcash * @factor),
        countsale = COALESCE(countsale, 0) + if(new.doctype = 1, 1, 0),
        countrefund = COALESCE(countrefund, 0) + if(new.doctype in (2, 25), 1, 0)
    WHERE workshiftid = new.workshiftid AND new.closed = 1;
END
"""
    exit
    trigger_after_update = """
CREATE TRIGGER triggerAfterUpdate AFTER UPDATE ON document
FOR EACH ROW BEGIN    
    set @oldfactor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);
    set @newfactor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    set @oldsumb=0;
    set @oldsumcash=0;    
    set @newsumb=0;    
    set @newsumcash=0;
    set @oldsumnoncash=0;
    set @newsumnoncash=0;
    set @oldcount=0;
    set @newcount=0;
    set @oldsumother=0;
    set @newsumother=0;

    IF old.closed = 1
    THEN
       set @oldsumb=old.sumb;
       set @oldsumother=old.sumother;
       set @oldsumcash=@oldfactor * old.sumcash;
       set @oldsumnoncash=@oldfactor * old.sumnoncash;
       set @oldcount=1;
    END IF;

    IF new.closed = 1 
    THEN
       set @newsumb=new.sumb;
       set @newsumother=new.sumother;
       set @newsumcash=@newfactor * new.sumcash;
       set @newsumnoncash=@newfactor * new.sumnoncash;
       set @newcount=1;
    END IF;        

    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, -@oldsumb + @newsumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), -@oldsumb + @newsumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - @oldsumcash + @newsumcash,        
        sumrefund = COALESCE(sumrefund, 0) + if(new.doctype in (2,25), -abs(@oldsumb) + abs(@newsumb), 0),
        sumsalecash = COALESCE(sumsalecash, 0) + if(new.doctype = 1, -@oldsumcash + @newsumcash, 0),        
        sumsalenoncash = COALESCE(sumsalenoncash, 0) + if(new.doctype = 1, -@oldsumnoncash + @newsumnoncash, 0),        
        sumsaleother = COALESCE(sumsaleother, 0) + if(new.doctype = 1, -@oldsumother + @newsumother, 0),        
        sumgaincash = COALESCE(sumgaincash, 0) + if(new.doctype in (1,2,25), -@oldsumcash + @newsumcash, 0),        
        sumgainnoncash = COALESCE(sumgainnoncash, 0) + if(new.doctype in (1,2,25), -@oldsumnoncash + @newsumnoncash, 0),        
        sumrefundcash = COALESCE(sumrefundcash, 0) + if(new.doctype in (2,25), -abs(@oldsumcash) + abs(@newsumcash), 0),        
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) + if(new.doctype in (2,25), -abs(@oldsumnoncash) + abs(@newsumnoncash), 0),        
        countsale = COALESCE(countsale, 0) + if(new.doctype = 1, -@oldcount + @newcount, 0),
        countrefund = COALESCE(countrefund, 0) + if(new.doctype in (2, 25), -@oldcount + @newcount, 0)
    WHERE workshiftid = new.workshiftid;
END
"""

    trigger_after_delete = """
CREATE TRIGGER triggerAfterDelete AFTER DELETE ON document
FOR EACH ROW BEGIN
    set @factor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);    
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) - if(old.doctype = 1, old.sumb, 0),
        sumgain = COALESCE(sumgain, 0) - if(old.doctype in (1, 2, 25), old.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - (old.sumcash * @factor),
        sumrefund = COALESCE(sumrefund, 0) - if(old.doctype in (2,25), abs(old.sumb), 0),        
        sumsalecash = COALESCE(sumsalecash, 0) - if(old.doctype = 1, old.sumcash, 0),                
        sumsalenoncash = COALESCE(sumsalenoncash, 0) - if(old.doctype = 1, old.sumnoncash, 0),              
        sumsaleother = COALESCE(sumsaleother, 0) - if(old.doctype = 1, old.sumother, 0),                
        sumgaincash = COALESCE(sumgaincash, 0) - if(old.doctype in (1,2,25), old.sumcash * @factor, 0),               
        sumgainnoncash = COALESCE(sumgainnoncash, 0) - if(old.doctype in (1,2,25), old.sumnoncash * @factor, 0),                
        sumrefundcash = COALESCE(sumrefundcash, 0) - if(old.doctype in (2,25), old.sumcash, 0),               
        sumrefundnoncash = COALESCE(sumrefundnoncash, 0) - if(old.doctype in (2,25), old.sumnoncash, 0),             
        countsale = COALESCE(countsale, 0) - if(old.doctype = 1, 1, 0),
        countrefund = COALESCE(countrefund, 0) - if(old.doctype in (2, 25), 1, 0)
    WHERE workshiftid = old.workshiftid AND old.closed = 1;       
END
"""
    migrate_engine.connect().execute(trigger_after_insert)
    migrate_engine.connect().execute(trigger_after_update)
    migrate_engine.connect().execute(trigger_after_delete)

def createOldTriggers(migrate_engine):
    trigger_after_insert = """
CREATE TRIGGER triggerAfterInsert AFTER INSERT ON document
FOR EACH ROW BEGIN
    SET @factor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, new.sumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), new.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) + (new.sumcash * @factor)
    WHERE workshiftid = new.workshiftid AND new.closed = 1;
END
"""
    exit
    trigger_after_update = """
CREATE TRIGGER triggerAfterUpdate AFTER UPDATE ON document
FOR EACH ROW BEGIN    
    set @oldfactor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);
    set @newfactor:=(select moneyfactor FROM documenttype WHERE doctype = new.doctype);
    set @oldsumb=0;
    set @oldsumcash=0;    
    set @newsumb=0;    
    set @newsumcash=0;   
    
    IF old.closed = 1
    THEN
       set @oldsumb=old.sumb;
       set @oldsumcash=@oldfactor * old.sumcash;
    END IF;
    
    IF new.closed = 1 
    THEN
       set @newsumb=new.sumb;
       set @newsumcash=@newfactor * new.sumcash;
    END IF;        
    
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) + if(new.doctype = 1, -@oldsumb + @newsumb, 0),
        sumgain = COALESCE(sumgain, 0) + if(new.doctype in (1, 2, 25), -@oldsumb + @newsumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - @oldsumcash + @newsumcash
    WHERE workshiftid = new.workshiftid;
END
"""
    
    trigger_after_delete = """
CREATE TRIGGER triggerAfterDelete AFTER DELETE ON document
FOR EACH ROW BEGIN
    set @factor:=(select moneyfactor FROM documenttype WHERE doctype = old.doctype);    
    UPDATE workshift 
    SET sumsale = COALESCE(sumsale, 0) - if(old.doctype = 1, old.sumb, 0),
        sumgain = COALESCE(sumgain, 0) - if(old.doctype in (1, 2, 25), old.sumb, 0),
        sumdrawer = COALESCE(sumdrawer, 0) - (old.sumcash * @factor)
    WHERE workshiftid = old.workshiftid AND old.closed = 1;       
END
"""
        
    migrate_engine.connect().execute(trigger_after_insert)
    migrate_engine.connect().execute(trigger_after_update)
    migrate_engine.connect().execute(trigger_after_delete)

def getWorkshiftTable(meta):
    return Table('workshift', meta, autoload=True)

def getDocumentTable(meta):
    return Table('document', meta, autoload=True)

def upgrade(migrate_engine):
    dump.store(migrate_engine)
    meta = MetaData(bind=migrate_engine)
    workshiftColumns = ['sumsalecash', 'sumsalenoncash', 'sumsaleother', 'sumgaincash', 'sumgainnoncash', 'sumrefund',
                        'sumrefundcash', 'sumrefundnoncash']
    for columnName in workshiftColumns:
        if columnName not in getWorkshiftTable(meta).columns:
            Column(columnName, Numeric(precision=13, scale=2), nullable=False).create(getWorkshiftTable(meta))

    for columnName in ['sumnoncash', 'sumother']:
        if columnName not in getDocumentTable(meta).columns:
            Column(columnName, Numeric(precision=13, scale=2), nullable=False).create(getDocumentTable(meta))

    for columnName in ['countsale', 'countrefund']:
        if columnName not in getWorkshiftTable(meta).columns:
            Column(columnName, Integer, nullable=False).create(getWorkshiftTable(meta))

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTriggers(migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createTriggers(migrate_engine)

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    if 'sumsalecash' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumsalecash.drop()
    if 'sumsalenoncash' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumsalenoncash.drop()
    if 'sumsaleother' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumsaleother.drop()
    if 'sumgaincash' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumgaincash.drop()
    if 'sumgainnoncash' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumgainnoncash.drop()
    if 'sumrefund' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumrefund.drop()
    if 'sumrefundcash' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumrefundcash.drop()
    if 'sumrefundnoncash' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.sumrefundnoncash.drop()
    if 'countsale' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.countsale.drop()
    if 'countrefund' in getWorkshiftTable(meta).columns:
        getWorkshiftTable(meta).c.countrefund.drop()
    if 'sumnoncash' in getDocumentTable(meta).columns:
        getDocumentTable(meta).c.sumnoncash.drop()
    if 'sumother' in getDocumentTable(meta).columns:
        getDocumentTable(meta).c.sumother.drop()

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        dropTriggers(migrate_engine)

    if isinstance(migrate_engine.connect().dialect, MySQLDialect):
        createOldTriggers(migrate_engine)
