from sqlalchemy import *
from migrate import *

organizationIdColumn = "organizationid"
organizationIdFk = "clientorganizationidfk"

def getTable(meta):
    return Table('organization', meta,
        Column('id', Integer, nullable=False, primary_key=True, autoincrement=True),
        Column('code', String(255), nullable=True),
        Column('name', String(255), nullable=False),
        Column('inn', String(255), nullable=False),
        mysql_engine='InnoDB'
    )

def getClient(meta):
    return Table('client', meta, autoload=True)

def upgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)
    table = getTable(meta)
    if not table.exists(meta.bind):
        table.create()

    client = getClient(meta)
    if organizationIdColumn not in client.columns:
        Column(organizationIdColumn, Integer, ForeignKey(table.c.id, name=organizationIdFk,
                ondelete='SET NULL'), nullable=True).create(client)

def downgrade(migrate_engine):
    meta = MetaData(bind=migrate_engine)

    table = getTable(meta)
    client = getClient(meta)
    if table.exists(meta.bind):
        if organizationIdFk in [key.name for key in client.foreign_keys]:
            changeset.constraint.ForeignKeyConstraint([client.c.organizationid], [table.c.id], name=organizationIdFk).drop()
        table.drop()
    if organizationIdColumn in client.columns:
        client.c.organizationid.drop(client)
    